#!/usr/bin/python
__author__ = 'Jiri_Kulda'


class InformationUnit(object):
    """
    This is main class containing nature language information
    :param inf: DocumentationInformation object with data
    """
    information = ""
    information_obj = ""

    def __init__(self, inf):
        self.information = ""
        self.information_obj = inf

    def set_information(self):
        pass

    def get_command_name(self):
        return self.information_obj.get_command_name()

    def connect_multiple_facts(self, facts, max_size=5):
        """
        This method makes more human language sentences by
        correct representing of word enumeration
        :param facts: list of words to enumerate
        :param max_size: Maximum size of words to be shown.
                         Default is 5.
        :return: set upped string line with enumerated words
        """
        pom_inf = ""
        if len(facts) == 1:
            pom_inf = facts[0]
        elif len(facts) == 2:
            pom_inf = facts[0] + " and " + facts[1]
        else:
            i = 0
            while i < max_size:
                pom_inf += facts[i]
                if len(facts) > (i + 2) and (i + 2) < max_size:
                    pom_inf += ", "
                elif (i + 1) == len(facts):
                    return pom_inf
                elif (i + 1) == max_size:
                    pom_inf += "..."
                    return pom_inf
                else:
                    pom_inf += " and "
                i += 1
            pom_inf += "..."
        return pom_inf

    def print_information(self):
        print("   " + self.information)

    def get_information_weigh(self):
        """
        This method calculates information weight.
        The weight is amount of lines on which information
        will be displayed

        :return: information weight
        """
        line_size = 60  # char per line
        weigh = (len(self.information)//line_size)
        mod_weigh = (len(self.information) % line_size)
        if weigh == 0:
            return 1
        elif mod_weigh >= 20:  # tolerance
            return weigh + 1
        else:
            return weigh

    def get_information_value(self):
        """
        :return: Return information importance
        """
        return self.information_obj.get_importance()

    def is_list_empty(self, tested_list):
        return len(tested_list) == 0

    def check_status_and_add_information(self, status):
        """
        This method replaces status number for better information string.
        :param status: command status
        """
        if not status == "-":
            if status == "1":
                self.information += " and must finished unsuccessfully"
            elif not status == "0":
                self.information += " and must finished with return code matching: " + status

    def set_correct_singulars_or_plurals(self, word, number_of_subject, ending="s", verb=False):
        """
        This method correctly represent singulars or plurals in word.
        :param word: word to set up
        :param number_of_subject: count of subjects in sentence
        :param ending: word ending in plural. default is "s"
        :param verb: possible verb after word
        :return: Correctly set upped word
        """
        pom_word = word
        if number_of_subject >= 2:
            if pom_word[-1] == "y" and ending == "ies":
                pom_word = word[:-1] + ending
            else:
                pom_word += ending

            if verb:
                pom_word += " are"
        elif verb:
            pom_word += " is"

        if not verb:
            pom_word += " "
        return pom_word


class InformationFileExists(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "File(directory): \"" + self.information_obj.get_topic_subject()[0] + "\""
        self.information += " must exist"
        self.check_status_and_add_information(self.information_obj.get_status())

    def check_status_and_add_information(self, status):
        """
        Overloaded method to correctly represent status in this case
        :param status: status number
        :return: correctly represented status number.
        """
        if status == "0":
            self.information = "File(directory): \"" + self.information_obj.get_topic_subject()[0] + "\" must exist"
        elif status == "1":
            self.information = "File(directory): \"" + self.information_obj.get_topic_subject()[0] + "\" must not exist"
        elif not status == "-":
            self.information += " and exit code must match " + status


class InformationFileNotExists(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "File(directory): \"" + self.information_obj.get_topic_subject()[0] + "\""
        self.information += " must not exist"
        self.check_status_and_add_information(self.information_obj.get_status())

    def check_status_and_add_information(self, status):
        """
        Overloaded method to correctly represent status in this case
        :param status: status number
        :return: correctly represented status number.
        """
        if status == "0":
            self.information = "File(directory): \"" + self.information_obj.get_topic_subject()[0] + "\" must not exist"
        elif status == "1":
            self.information = "File(directory): \"" + self.information_obj.get_topic_subject()[0] + "\" must exist"
        elif not status == "-":
            self.information += " and exit code must match " + status


class InformationFileContain(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "File: \"" + self.information_obj.get_topic_subject()[0] \
                           + "\" must contain pattern: \"" + self.information_obj.get_topic_subject()[1] + "\""
        self.check_status_and_add_information(self.information_obj.get_status())

    def check_status_and_add_information(self, status):
        """
        Overloaded method to correctly represent status in this case
        :param status: status number
        :return: correctly represented status number.
        """
        if status == "0":
            self.information = "File: \"" + self.information_obj.get_topic_subject()[0] \
                               + "\" must contain pattern: \"" + self.information_obj.get_topic_subject()[1] + "\""
        elif status == "1":
            self.information = "File: \"" + self.information_obj.get_topic_subject()[0] \
                               + "\" must not contain pattern: \"" + self.information_obj.get_topic_subject()[1] + "\""
        elif not status == "-":
            self.information += " and exit code must match " + status


class InformationFileNotContain(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "File: \"" + self.information_obj.get_topic_subject()[0] \
                           + "\" must not contain pattern \"" + self.information_obj.get_topic_subject()[1]
        self.information += "\""
        self.check_status_and_add_information(self.information_obj.get_status())

    def check_status_and_add_information(self, status):
        """
        Overloaded method to correctly represent status in this case
        :param status: status number
        :return: correctly represented status number.
        """
        if status == "0":
            self.information = "File: \"" + self.information_obj.get_topic_subject()[0] \
                               + "\" must not contain pattern: \"" + self.information_obj.get_topic_subject()[1] + "\""
        elif status == "1":
            self.information = "File: \"" + self.information_obj.get_topic_subject()[0] \
                               + "\" must contain pattern: \"" + self.information_obj.get_topic_subject()[1] + "\""
        elif not status == "-":
            self.information += " and exit code must match " + status


class InformationJournalPrint(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "Prints the content of the journal in pretty " + self.information_obj.get_topic_subject()[0]
        self.information += " format"
        if len(self.information_obj.get_option()):
            self.information += " with additional information"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationPackagePrint(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "Shows information about "
        self.information += self.connect_multiple_facts(self.information_obj.get_topic_subject(), 4)
        self.information += " version"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileResolve(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Resolves absolute path " + subjects[0]
        if len(subjects) == 3:
            self.information += ", replaces / for " + subjects[1]
            self.information += " and rename file to " + subjects[2]
        else:
            self.information += " and replaces / for " + subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileCreate(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subject = self.information_obj.get_topic_subject()
        self.information = "Creates a tarball of " + self.set_correct_singulars_or_plurals("file", len(subject))
        self.information += self.connect_multiple_facts(subject, 3)
        self.information += " and attached it/them to test result"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationMessageCreate(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        option = self.information_obj.get_option()
        if subjects[0] == "kernel":  # rlShowRunningKernel
            self.information = "Log a message with version of the currently running kernel"
        else:  # rlDie & rlLog
            self.information = "Message \"" + subjects[0]
            if len(subjects) > 1:
                self.information += "\" is created in to log and "
                self.information += self.set_correct_singulars_or_plurals("file", len(subjects[1:]))
                self.information += self.connect_multiple_facts(subjects[1:], 3)
                if len(subjects[1:]) > 1:
                    self.information += "\" are uploaded"
                else:
                    self.information += "\" is uploaded"
            else:
                if not self.is_list_empty(option):
                    self.information += "\" is created in to logfile "
                    self.information += option[0]
                else:
                    self.information += "\" is created in to log"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFilePrint(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        if self.information_obj.get_topic_subject()[0] == "makefile":
            self.information = "Prints comma separated list of requirements defined in Makefile"
        else:
            self.information = "Prints file content"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileCheck(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        if self.information_obj.get_topic_subject()[0] == "makefile":
            self.information = "Checks requirements in Makefile and returns number of compliance"
        else:
            self.information = "Checks file " + self.information_obj.get_topic_subject()[0]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationJournalReturn(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        if subjects[0] == "phase":
            self.information = "Returns number of failed asserts in current phase"
        elif subjects[0] == "test":
            self.information = "Returns number of failed asserts"
        elif subjects[0] == "variant":
            self.information = "Returns variant of the distribution on the system"
        elif subjects[0] == "release":
            self.information = "Returns release of the distribution on the system"
        elif subjects[0] == "primary":
            self.information = "Returns primary arch for the current system"
        elif subjects[0] == "secondary":
            self.information = "Returns base arch for the current system"
        else:
            self.information = "Returns data from Journal"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationCommandRun(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        if subjects[0] == "watchdog":
            self.information = "Runs command " + subjects[1]
            self.information += " for " + subjects[2]
            self.information += " seconds"
            if not self.is_list_empty(self.information_obj.get_option()):
                self.information += " and killed with signal "
                self.information += self.information_obj.get_option()[0]

        else:  # rlRun
            self.information = "Command \"" + subjects[0]
            if subjects[1] == "0":
                self.information += "\" must run successfully"
            elif subjects[1] == "1":
                self.information += "\" must run unsuccessfully"
            else:
                self.information += "\" exit code must match: " + subjects[1]

            option = self.information_obj.get_option()
            if not self.is_list_empty(option):
                if option[0] == "l":
                    self.information += " and output is stored in to log"
                elif option[0] == "c":
                    self.information += " and failed output is stored in to log"
                elif len(option) > 1:
                    self.information += " and stdout and stderr are tagged and stored"
                elif option[0] == "t":
                    self.information += " and stdout and stderr are tagged"
                elif option[0] == "s":
                    self.information += " and stdout and stderr are stored"


class InformationServerRun(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "Starts virtual X " + self.information_obj.get_topic_subject()[0] + \
                           " server on a first free display"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationServerKill(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "Kills virtual X " + self.information_obj.get_topic_subject()[0] + " server"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationServerReturn(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "Shows number of displays where virtual X " + self.information_obj.get_topic_subject()[0] + \
                           " is running"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationJournalReport(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Reports test \"" + subjects[0]
        self.information += "\" with result " + subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationCommandWait(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        if subjects[0] == "cmd":
            # rlWaitForCmd
            option = self.information_obj.get_option()
            self.information = "Pauses script execution until command " + subjects[1]
            if option[0] == "0":
                self.information += " exit status is successful"
            elif option[0] == "1":
                self.information += " exit status is unsuccessful"
            else:
                self.information += " exit status match " + option[0]

            if option[1]:
                self.information += "\n and process with this PID " + option[1]
                self.information += " must be running"
        else:  # rlWait
            if subjects:
                self.information = "Wait for " + self.connect_multiple_facts(subjects, 3)
            else:
                self.information = "All currently active child processes are"
                self.information += " waited for, and the return status is zero"
            self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileWait(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        option = self.information_obj.get_option()
        if self.information_obj.get_topic_subject()[0] == "file":  # rlWaitForFile
            if not self.is_list_empty(option):
                self.information = "Pauses script until file or directory with this path "
                self.information += self.information_obj.get_topic_subject()[1] + " starts existing"
                self.information += "\n and process with this PID " + option[0]
                self.information += " must be running"
            else:
                self.information = "Pauses script until file or directory with this path "
                self.information += self.information_obj.get_topic_subject()[1] + " starts listening"
        else:  # rlWaitForScript
            if not self.is_list_empty(option):
                if option[0] == "close":
                    self.information = "Waits for the socket with this path"
                    self.information += self.information_obj.get_topic_subject()[1] + "to stop listening"
                elif option[0] == "p":
                    self.information = "Pauses script until socket with this path or port "
                    self.information += self.information_obj.get_topic_subject()[1] + " starts listening"
                    self.information += "\n and process with this PID " + option[0]
                    self.information += " must be running"
            else:
                self.information = "Pauses script until socket with this path or port "
                self.information += self.information_obj.get_topic_subject()[1] + " starts listening"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationPackageImport(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subject = self.information_obj.get_topic_subject()
        self.information = "Imports code provided by "
        self.information += self.connect_multiple_facts(subject, 2)
        self.information += self.set_correct_singulars_or_plurals(" library", len(subject), "ies")
        self.information += "  library(ies) into the test namespace"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationCommandMeasures(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        option = self.information_obj.get_option()
        if not self.is_list_empty(option):
            self.information = "Measures, how many runs of command "
            self.information += subjects[0] + " in "
            self.information += option[0] + " second(s)"
        else:
            self.information = "Measures the average time of running command "
            self.information += subjects[0]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationStringCreate(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        if self.information_obj.get_topic_subject()[0] == "append":
            self.information = "Appends string: " + self.information_obj.get_topic_subject()[1]
            self.information += " to the cleanup buffer"
            self.information += " and recreates the cleanup script"
        else:
            self.information = "Prepends string: " + self.information_obj.get_topic_subject()[0]
            self.information += " to the cleanup buffer"
            self.information += " and recreates the cleanup script"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationBooleanSet(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        if subjects[0] == "on":
            self.information = "Sets "
            self.information += self.set_correct_singulars_or_plurals("boolean", len(subjects[1:]))
            self.information += self.connect_multiple_facts(subjects[1:], 3)
            self.information += " to true"
        elif subjects[0] == "off":
            self.information = "Sets "
            self.information += self.set_correct_singulars_or_plurals("boolean", len(subjects[1:]))
            self.information += self.connect_multiple_facts(subjects[1:], 3)
            self.information += " to false"
        else:
            self.information = "Restore "
            self.information += self.set_correct_singulars_or_plurals("boolean", len(subjects))
            self.information += self.connect_multiple_facts(subjects, 3)
            self.information += " into original state"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationServiceRun(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subject = self.information_obj.get_topic_subject()
        self.information = "Starts "
        self.information += self.set_correct_singulars_or_plurals("service", len(subject))
        self.information += self.connect_multiple_facts(subject, 3)
        self.check_status_and_add_information(self.information_obj.get_status())

    def check_status_and_add_information(self, status):
        """self.information[-1] is here because method set_correct_singulars... adds one space on the end of the word"""

        subject = self.information_obj.get_topic_subject()
        if status == "0":
            self.information = self.set_correct_singulars_or_plurals("Service", len(subject))
            self.information = self.information[:-1] + ": " + self.connect_multiple_facts(subject, 3) + \
                               " must be running"
        elif status == "1":
            self.information = self.set_correct_singulars_or_plurals("Service", len(subject))
            self.information = self.information[:-1] + ": " + self.connect_multiple_facts(subject, 3) + \
                               " must not be running"
        elif not status == "-":
            self.information += " and exit code must match " + status


class InformationServiceKill(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subject = self.information_obj.get_topic_subject()
        self.information = "Kills "
        self.information += self.set_correct_singulars_or_plurals("service", len(subject))
        self.information += self.connect_multiple_facts(subject, 3)
        self.check_status_and_add_information(self.information_obj.get_status())

    def check_status_and_add_information(self, status):
        """self.information[-1] is here because method set_correct_singulars... adds one space on the end of the word"""
        subject = self.information_obj.get_topic_subject()
        if status == "0":
            self.information = self.set_correct_singulars_or_plurals("Service", len(subject))
            self.information = self.information[:-1] + ": " + self.connect_multiple_facts(self.information_obj.get_topic_subject(), 3) + \
                               " must not be running"
        elif status == "1":
            self.information = self.set_correct_singulars_or_plurals("Service", len(subject))
            self.information = self.information[:-1] + ": " + self.connect_multiple_facts(self.information_obj.get_topic_subject(), 3) + \
                               " must be running"
        elif not status == "-":
            self.information += " and exit code must match " + status


class InformationServiceRestore(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subject = self.information_obj.get_topic_subject()
        self.information = self.set_correct_singulars_or_plurals("Service", len(subject))
        self.information += self.connect_multiple_facts(subject, 3)
        if len(subject) > 1:
            self.information += " are restored into original state"
        else:
            self.information += " is restored into original state"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileRestore(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        option = self.information_obj.get_option()
        if not self.is_list_empty(option):
            self.information = "Restores backed up file with namespace: "
            self.information += option[0]
            self.information += "to original state"
        else:
            self.information = "Restores backed up files to their "
            self.information += "original location"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileBackup(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        option = self.information_obj.get_option()
        status = self.information_obj.get_status()
        subject = self.information_obj.get_topic_subject()
        self.information = self.set_correct_singulars_or_plurals("File", len(subject))
        self.information += "or " + self.set_correct_singulars_or_plurals("directory", len(subject), "ies")
        self.information += self.connect_multiple_facts(subject, 2)
        if len(subject) >= 2:
            self.information += " are backed up"
        else:
            self.information += " is backed up"
        if not self.is_list_empty(option):
            self.information += "with namespace " + option[0]
        self.check_status_and_add_information(status)


class InformationStringHash(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        option = self.information_obj.get_option()
        self.information = "Hashes string "
        if subjects[0] == True:
            self.information += "from input"
        else:
            self.information += subjects[0]
        if not self.is_list_empty(option):
            self.information += " with hashing algorithm "
            self.information += option[0]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationStringUnHash(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        option = self.information_obj.get_option()
        subjects = self.information_obj.get_topic_subject()
        self.information = "Unhashes string "
        if subjects[0] ==  True:
            self.information += "from input"
        else:
            self.information += subjects[0]

        if not self.is_list_empty(option):
            self.information += " with hashing algorithm "
            self.information += option[0]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationMountpointExists(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Directory "
        self.information += subjects[0]
        self.information += "must be a mountpoint"

        if subjects[1]:
            self.information += " to server " + subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationMountpointCreate(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Creates mount point " + subjects[0]
        if subjects[1]:
            self.information += " and mount NFS " + subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationMountpointCheck(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Checks if directory "
        self.information += subjects[0]
        self.information += "is a mountpoint"

        if subjects[1]:
            self.information += " to server " + subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationPackageOwnedBy(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Binary " + subjects[0] + "must be"
        self.information += " owned by "
        self.information += self.set_correct_singulars_or_plurals("package", len(subjects[1:]))
        self.information += self.connect_multiple_facts(subjects[1:], 4)
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationSystemIsRHEL(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information += "Checks if we are running on"
        self.information += " RHEL "
        if subjects:
            self.information += self.connect_multiple_facts(subjects)
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationSystemIsFedora(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information += "Checks if we are running on"
        self.information += " Fedora "
        if subjects:
            self.information += self.connect_multiple_facts(subjects)
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileDiffer(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "File1 " + subjects[0] + " and file2 "
        self.information += subjects[1]
        self.information += " must be different"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationFileNotDiffer(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "File1 " + subjects[0] + " and file2 "
        self.information += subjects[1]
        self.information += " must be identical"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationValueEqual(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Value1 " + subjects[0]
        self.information += " must be equal to value2 "
        self.information += subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationValueNotEqual(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Value1 " + subjects[0]
        self.information += " must not be equal to value2 "
        self.information += subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationValueGreater(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Value1 " + subjects[0]
        self.information += " must be greater than value2 "
        self.information += subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationValueGreaterOrEqual(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        self.information = "Value1 " + subjects[0]
        self.information += " must be greater or equal to value2 "
        self.information += subjects[1]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationValueCheck(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        self.information = "Value " + self.information_obj.get_topic_subject()[0] + " must be 0"
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationPackageCheck(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        option = self.information_obj.get_option()
        self.information = "Package " + self.information_obj.get_topic_subject()[0]
        self.information += " must be installed"

        if not self.is_list_empty(option):
            self.information += " with"
            if len(option) == 1:
                self.information += " version: " + option[0]

            elif len(option) == 2:
                self.information += " version: " + option[0]
                self.information += " and release: " + option[1]

            else:
                self.information += " version: " + option[0]
                self.information += ", release: " + option[1]
                self.information += " and architecture: " + option[2]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationPackageExists(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        subjects = self.information_obj.get_topic_subject()
        option = self.information_obj.get_option()
        if subjects[0] == "all":
            self.information = "Packages in $PACKAGES, $REQUIRES"
            self.information += " and $COLLECTIONS must be installed"
        else:
            self.information = "Package " + subjects[0]
            self.information += " must be installed"

        if not self.is_list_empty(option):
            self.information += " with"
            if len(option) == 1:
                self.information += " version: " + option[0]

            elif len(option) == 2:
                self.information += " version: " + option[0]
                self.information += " and release: " + option[1]

            else:
                self.information += " version: " + option[0]
                self.information += ", release: " + option[1]
                self.information += " and architecture: " + option[2]
        self.check_status_and_add_information(self.information_obj.get_status())


class InformationPackageNotExists(InformationUnit):
    """
    Small InformationUnit class which contains information in human language.
    """
    def set_information(self):
        """
        Sets nature language information. This setting depends on small
        InformationUnit class.
        """
        option = self.information_obj.get_option()
        self.information = "Package " + self.information_obj.get_topic_subject()[0]
        self.information += " must not be installed"

        if not self.is_list_empty(option):
            self.information += " with"
            if len(option) == 1:
                self.information += " version: " + option[0]

            elif len(option) == 2:
                self.information += " version: " + option[0]
                self.information += " and release: " + option[1]

            else:
                self.information += " version: " + option[0]
                self.information += ", release: " + option[1]
                self.information += " and architecture: " + option[2]
        self.check_status_and_add_information(self.information_obj.get_status())



#!/usr/bin/python
__author__ = 'Jiri_Kulda'

import shlex

class TestFunction:
    """Class for working with functions from the BeakerLib test"""

    statement_list = []

    data_list = []

    name = ""

    def __init__(self, fname):
        self.statement_list = []
        lex = shlex.shlex(fname)
        self.name = lex.get_token()
        self.data_list = []

    def add_line(self, line):
        self.statement_list.append(line)

    def add_data(self, data):
        self.data_list.append(data)

    def is_function_data_empty(self):
        return len(self.data_list) == 0


#!/usr/bin/python
__author__ = 'Jiri_Kulda'
from bkrdoc.analysis import *


class GetInformation(object):
    """
    This class is responsible for transformation from DocumentationInformation class
    to small InformationUnit class. This class contain big 2D array with reference to
    small InformationUnit classes.
    """
    array = [
        # topic: FILE(DIRECTORY),           STRING                   PACKAGE          JOURNAL,PHASE,TEST   MESSAGE         COMMAND                SERVER              BOOLEAN              SERVICE            MOUNTPOINT              SYSTEM                 VALUE  # ACTIONS
        [InformationFileExists,           0,               InformationPackageExists,              0,              0,              0,                     0,                  0,                  0,  InformationMountpointExists,           0,                     0],  # exists
        [InformationFileNotExists,        0,           InformationPackageNotExists,           0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # not exists
        [InformationFileContain,          0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # contain
        [InformationFileNotContain,       0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # not contain
        [InformationFilePrint,            0,           InformationPackagePrint,   InformationJournalPrint,    0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # print(show)
        [InformationFileResolve,          0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # resolve
        [InformationFileCreate, InformationStringCreate,          0,                          0, InformationMessageCreate,    0,                     0,                  0,                  0,  InformationMountpointCreate,           0,                     0],  # create
        [InformationFileCheck,            0,           InformationPackageCheck,               0,              0,              0,                     0,                  0,                  0,  InformationMountpointCheck,            0,         InformationValueCheck],  # check
        [           0,                    0,                      0,                InformationJournalReturn, 0,              0,         InformationServerReturn,        0,                  0,                   0,                    0,                     0],  # return
        [           0,                    0,                      0,                          0,              0,  InformationCommandRun, InformationServerRun,           0,       InformationServiceRun,          0,                    0,                     0],  # run
        [           0,                    0,                      0,                InformationJournalReport, 0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # report
        [           0,                    0,                      0,                          0,              0,              0,          InformationServerKill,         0,       InformationServiceKill,         0,                    0,                     0],  # kill
        [InformationFileWait,             0,                      0,                          0,              0,  InformationCommandWait,            0,                  0,                  0,                   0,                    0,                     0],  # wait
        [           0,                    0,           InformationPackageImport,              0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # import
        [           0,                    0,                      0,                          0,              0,  InformationCommandMeasures,        0,                  0,                  0,                   0,                    0,                     0],  # measures
        [           0,                    0,                      0,                          0,              0,              0,                     0,     InformationBooleanSet,           0,                   0,                    0,                     0],  # set
        [InformationFileRestore,          0,                      0,                          0,              0,              0,                     0,                  0,      InformationServiceRestore,       0,                    0,                     0],  # restore
        [InformationFileBackup,           0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # backup
        [           0,          InformationStringHash,            0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # hash
        [           0,          InformationStringUnHash,          0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # unhash
        [           0,                    0,           InformationPackageOwnedBy,             0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # owned by
        [           0,                    0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,    InformationSystemIsRHEL,               0],  # is RHEL
        [           0,                    0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,   InformationSystemIsFedora,              0],  # is Fedora
        [InformationFileDiffer,           0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # differ
        [InformationFileNotDiffer,        0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,                     0],  # not differ
        [           0,                    0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,         InformationValueEqual],  # equal
        [           0,                    0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,       InformationValueNotEqual],  # not equal
        [           0,                    0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,        InformationValueGreater],  # greater
        [           0,                    0,                      0,                          0,              0,              0,                     0,                  0,                  0,                   0,                    0,   InformationValueGreaterOrEqual],  # greater or equal
    ]

    def get_information_from_facts(self, information_obj):
        """
        Method responsible for the transformation
        :param information_obj: DocumentationInformation instanse to be transformed
        :return: InformationUnit class or empty string.
        """
        information = ""
        topic = information_obj.get_topic()
        for action in information_obj.get_action():
            column = self.get_topic_number(topic)
            row = self.get_action_number(action)
            information_class = self.array[row][column]
            if information_class:
                information = information_class(information_obj)
                information.set_information()
        return information

    def get_action_number(self, action):
        """
        method responsible for returning the right row number
        :param action: action word
        :return: row number
        """
        if self.is_action_exists(action):
            return 0
        elif self.is_action_not_exists(action):
            return 1
        elif self.is_action_contain(action):
            return 2
        elif self.is_action_not_contain(action):
            return 3
        elif self.is_action_print(action):
            return 4
        elif self.is_action_resolve(action):
            return 5
        elif self.is_action_create(action):
            return 6
        elif self.is_action_check(action):
            return 7
        elif self.is_action_return(action):
            return 8
        elif self.is_action_run(action):
            return 9
        elif self.is_action_report(action):
            return 10
        elif self.is_action_kill(action):
            return 11
        elif self.is_action_wait(action):
            return 12
        elif self.is_action_import(action):
            return 13
        elif self.is_action_measures(action):
            return 14
        elif self.is_action_set(action):
            return 15
        elif self.is_action_restore(action):
            return 16
        elif self.is_action_backup(action):
            return 17
        elif self.is_action_hash(action):
            return 18
        elif self.is_action_unhash(action):
            return 19
        elif self.is_action_owned_by(action):
            return 20
        elif self.is_action_is_rhel(action):
            return 21
        elif self.is_action_is_fedora(action):
            return 22
        elif self.is_action_differ(action):
            return 23
        elif self.is_action_not_differ(action):
            return 24
        elif self.is_action_equal(action):
            return 25
        elif self.is_action_not_equal(action):
            return 26
        elif self.is_action_greater(action):
            return 27
        elif self.is_action_greater_or_equal(action):
            return 28

    def get_topic_number(self, topic):
        """
        method responsible for returning the right topic column number
        :param topic: Topic word
        :return: column
        """
        if self.is_topic_file(topic):
            return 0
        elif self.is_topic_string(topic):
            return 1
        elif self.is_topic_package(topic):
            return 2
        elif self.is_topic_journal(topic):
            return 3
        elif self.is_topic_message(topic):
            return 4
        elif self.is_topic_command(topic):
            return 5
        elif self.is_topic_server(topic):
            return 6
        elif self.is_topic_boolean(topic):
            return 7
        elif self.is_topic_service(topic):
            return 8
        elif self.is_topic_mountpoint(topic):
            return 9
        elif self.is_topic_system(topic):
            return 10
        elif self.is_topic_value(topic):
            return 11

    def is_topic_file(self, topic):
        return topic == "FILE"

    def is_topic_string(self, topic):
        return topic == "STRING"

    def is_topic_package(self, topic):
        return topic == "PACKAGE"

    def is_topic_journal(self, topic):
        return topic == "JOURNAL"

    def is_topic_message(self, topic):
        return topic == "MESSAGE"

    def is_topic_command(self, topic):
        return topic == "COMMAND"

    def is_topic_server(self, topic):
        return topic == "SERVER"

    def is_topic_boolean(self, topic):
        return topic == "BOOLEAN"

    def is_topic_service(self, topic):
        return topic == "SERVICE"

    def is_topic_mountpoint(self, topic):
        return topic == "MOUNTPOINT"

    def is_topic_system(self, topic):
        return topic == "SYSTEM"

    def is_topic_value(self, topic):
        return topic == "VALUE"

    def is_action_exists(self, action):
        return action == "exists"

    def is_action_not_exists(self, action):
        return action == "not exists"

    def is_action_contain(self, action):
        return action == "contain"

    def is_action_not_contain(self, action):
        return action == "not contain"

    def is_action_print(self, action):
        return action == "print"

    def is_action_resolve(self, action):
        return action == "resolve"

    def is_action_create(self, action):
        return action == "create"

    def is_action_check(self, action):
        return action == "check"

    def is_action_return(self, action):
        return action == "return"

    def is_action_run(self, action):
        return action == "run"

    def is_action_report(self, action):
        return action == "report"

    def is_action_kill(self, action):
        return action == "kill"

    def is_action_wait(self, action):
        return action == "wait"

    def is_action_import(self, action):
        return action == "import"

    def is_action_measures(self, action):
        return action == "measures"

    def is_action_set(self, action):
        return action == "set"

    def is_action_restore(self, action):
        return action == "restore"

    def is_action_backup(self, action):
        return action == "backup"

    def is_action_hash(self, action):
        return action == "hash"

    def is_action_unhash(self, action):
        return action == "unhash"

    def is_action_owned_by(self, action):
        return action == "owned_by"

    def is_action_is_rhel(self, action):
        return action == "RHEL"

    def is_action_is_fedora(self, action):
        return action == "Fedora"

    def is_action_differ(self, action):
        return action == "differ"

    def is_action_not_differ(self, action):
        return action == "not differ"

    def is_action_equal(self, action):
        return action == "equal"

    def is_action_not_equal(self, action):
        return action == "not equal"

    def is_action_greater(self, action):
        return action == "greater"

    def is_action_greater_or_equal(self, action):
        return action == "greater or equal"


from distutils.core import setup

setup(
    name='bkrdoc',
    version='1.2.4',
    packages=['bkrdoc', 'bkrdoc.markup', 'bkrdoc.analysis'],
    url='https://github.com/rh-lab-q/bkrdoc',
    license='BSD',
    author='Jiri Kulda',
    author_email='Kulda12@seznam.cz, jkulda@redhat.com',
    install_requires=['*python*>=2.7'],
    description='This project aims to provide tools for automated documentation generation for BeakerLib tests. ',
    classifiers=["Programming Language :: Python",
                 "Programming Language :: Python :: 3",
                 "License :: OSI Approved :: BSD License",
                 "Development Status :: 4 - Beta",
                 "Topic :: Software Development :: Testing",
                 "Topic :: Software Development :: Documentation",
                 "Operating System :: OS Independent",
                 "Intended Audience :: Developers"]
)



__author__ = 'jkulda'

from .option_object import *
from .test_variables import *
from .test_functions import *
from .information_units import *
from .conditions_for_commands import *
from .topic_object import *
from .bkrdoc_parser import *
from .credibility import *
from .data_containers import *
from .doc_information_representation import *
from .documentation_translator import *
from .documentation_generator import *
from .get_information import *
from .Statement_data_searcher import *



__author__ = 'jkulda'

import shlex
from bkrdoc.markup.data_containers import *


class Parser(object):

    phases = []

    def __init__(self):
        self.phases = []

    def parse_file_by_lines(self, file_string):
        tagged_comment_container = ""
        self.phases.append(PhaseOutsideContainer())
        splitted_file_string = file_string.split("\n")
        max_len = len(splitted_file_string)
        i = 0
        while i < max_len:
            line = splitted_file_string[i]
            line = line.strip()
            splitted_line = shlex.split(line, posix=False)

            if not self.is_empty_line(splitted_line):
                if self.is_phase_start_xxx(splitted_line[0]):
                    self.phases.append(TestPhaseContainer())

                elif self.is_phase_journal_end(splitted_line[0]) or self.is_journal_start(splitted_line[0]):
                    if self.is_tagged_comment_container(tagged_comment_container):
                        self.phases[-1].add_comment(tagged_comment_container)
                        tagged_comment_container = ""
                    self.phases.append(PhaseOutsideContainer())

            i, tagged_comment_container, container = self.comments_and_containers_parsing(splitted_line,
                                                                                          tagged_comment_container,
                                                                                          self.phases[-1],
                                                                                          splitted_file_string,
                                                                                          i, line)
            i += 1

    def comments_and_containers_parsing(self, splitted_line, tagged_comment_container, container, splitted_file_string,
                                        i, line):

        if not self.is_empty_line(splitted_line) and self.is_tagged_comment_start(splitted_line) and \
           not self.is_tagged_comment_container(tagged_comment_container):
                tagged_comment_container = TaggedCommentContainer(splitted_line)
        elif not self.is_empty_line(splitted_line) and self.is_common_comment_line(splitted_line):
            if self.is_tagged_comment_container(tagged_comment_container):
                # print splitted_line
                tagged_comment_container.add_comment(splitted_line)
            else:
                container.add_common_comment(splitted_line)
        else:
            if not self.is_empty_line(splitted_line):
                if self.is_condition_line(splitted_line):
                    i, tagged_comment_container, condition = self.specific_container_parse_call(i, tagged_comment_container, splitted_line,
                                                                      ConditionContainer(), ["fi"],
                                                                      splitted_file_string)
                    container.add_statement_line(condition)
                    container.add_comment(condition)
                    tagged_comment_container = ""
                elif self.is_loop_line(splitted_line):
                    loop = LoopContainer()
                    i, tagged_comment_container, loop = self.specific_container_parse_call(i, tagged_comment_container, splitted_line,
                                                                 loop, ["done"], splitted_file_string)
                    container.add_statement_line(loop)
                    container.add_comment(loop)
                    tagged_comment_container = ""

                elif self.is_function_line(splitted_line):
                    function = FunctionContainer()
                    i, tagged_comment_container, function = self.specific_container_parse_call(i, tagged_comment_container, splitted_line,
                                                                     function, ["}"], splitted_file_string)
                    container.add_comment(function)
                    container.add_statement_line(function)
                    tagged_comment_container = ""

                else:
                    container.add_statement_line(line)
                if self.is_tagged_comment_container(tagged_comment_container):
                    tagged_comment_container.add_tagged_line(splitted_line)
                    container.add_comment(tagged_comment_container)
                    tagged_comment_container = ""
            else:
                if self.is_tagged_comment_container(tagged_comment_container):
                    container.add_comment(tagged_comment_container)
                    tagged_comment_container = ""
            if not self.is_empty_line(splitted_line) and self.is_important_code_line(splitted_line):
                tagged_comment_container = TaggedCommentContainer([""])
                tagged_comment_container.add_condition_tag("code")
                tagged_comment_container.add_tagged_line(splitted_line)
                container.add_comment(tagged_comment_container)
                tagged_comment_container = ""

        return i, tagged_comment_container, container

    def parse_condition(self, i, splitted_file_string, searched_last_keyword, container):
        max_len = len(splitted_file_string)
        container.add_statement_line(splitted_file_string[i].strip())
        i += 1
        tagged_comment_container = ""
        is_elif_condition_part = False

        while i < max_len:
            line = splitted_file_string[i]
            line = line.strip()
            splitted_line = shlex.split(line, posix=False)

            if not self.is_empty_line(splitted_line) and self.is_seacher_keyword(searched_last_keyword, splitted_line):
                if self.is_tagged_comment_container(tagged_comment_container):
                    tagged_comment_container.add_tagged_line(splitted_line)
                    # container.add_comment(tagged_comment_container)
                    # print "first {0}".format(tagged_comment_container.comments)
                return i, tagged_comment_container, container

            elif not self.is_empty_line(splitted_line) and self.is_seacher_keyword(["else", "elif", "fi"], splitted_line):
                i, tagged_comment_container, condition = self.specific_container_parse_call(i, tagged_comment_container, splitted_line,
                                                                  ConditionContainer(), ["else", "elif", "fi"],
                                                                  splitted_file_string)
                # if self.is_tagged_comment_container(tagged_comment_container):
                #   print "second {0}".format(tagged_comment_container.comments)
                container.add_elif_part(condition)
                line = splitted_file_string[i]
                line = line.strip()
                # print container.elif_parts
                #tagged_comment_container = ""
                splitted_line = shlex.split(line, posix=False)
                # print "splitted_line {0} last keywords {1}".format(splitted_line, searched_last_keyword)
                if not self.is_empty_line(splitted_line) and self.is_seacher_keyword(searched_last_keyword, splitted_line):
                    if self.is_tagged_comment_container(tagged_comment_container):
                        container.add_comment(tagged_comment_container)
                        # print "second {0}".format(tagged_comment_container.comments)
                    # print "AHOOOJ"
                    return i, tagged_comment_container, container
                # print "elif"
                is_elif_condition_part = True
                i -= 1

            # print "impossible"
            if not is_elif_condition_part:
                # print "ZDE"
                i, tagged_comment_container, container = self.comments_and_containers_parsing(splitted_line,
                                                                                              tagged_comment_container,
                                                                                              container,
                                                                                              splitted_file_string,
                                                                                              i, line)
            if is_elif_condition_part:
                is_elif_condition_part = False
            i += 1
        return i, tagged_comment_container, container

    def specific_container_parse_call(self, i, tagged_comment_container, splitted_line, specific_container,
                                      end_list, splitted_file_string):
        if self.is_tagged_comment_container(tagged_comment_container):
            tagged_comment_container.add_tagged_line(splitted_line)
            specific_container.add_comment(tagged_comment_container)
            return self.parse_condition(i, splitted_file_string, end_list, specific_container)
        else:
            return self.parse_condition(i, splitted_file_string, end_list, specific_container)

    def get_phases(self):
        return self.phases

    def comments_set_up(self):
        for phase in self.phases:
            phase.comments_set_up()

    def is_important_code_line(self, splitted_line):
        return splitted_line[-1].endswith("#@")

    def is_seacher_keyword(self, searched_keyword, line):
        return line[0] in searched_keyword

    def is_tagged_comment_start(self, splitted_line):
        if splitted_line[0].startswith("#@"):
            return True
        elif len(splitted_line) > 1 and splitted_line[0] == "#" and splitted_line[1] == "@":
            return True
        else:
            return False

    def is_function_line(self, splitted_line):
        return splitted_line[0] == "function"

    def is_common_comment_line(self, line):
        return line[0].startswith("#")

    def is_empty_line(self, line):
        return line == []

    def is_phase_start_xxx(self, word):
        phase_list = ["rlPhaseStartSetup", "rlPhaseStartCleanup", "rlPhaseStartTest"]
        return word in phase_list

    def is_journal_start(self, word):
        return word == "rlJournalStart"

    def is_phase_journal_end(self, line):
        if line[0:len("rlphaseend")].lower() == "rlphaseend":
            return True

        elif line[0:len("rljournalend")].lower() == "rljournalend":
            return True

        else:
            return False

    def is_condition_line(self, line):
        return line[0] == "if"

    def is_loop_line(self, line):
        loop_list = ["for", "while", "until"]
        return line[0] in loop_list

    def is_tagged_comment_container(self, tagged_container):
        return type(tagged_container).__name__ == "TaggedCommentContainer"

    def is_phase_outside_container(self, container):
        return type(container).__name__ == "PhaseOutsideContainer"

    def is_phase_startxxx_container(self, container):
        return type(container).__name__ == "TestPhaseContainer"


